
#include <FastLED.h>
#include <Wire.h>
#include "pitches.h"
#include "SparkFun_CAP1203.h"

// Set up the PIN
#define BUTTON_RED_PIN 8
#define BUTTON_BLUE_PIN 12
#define BUTTON_GREEN_PIN 13
#define LED_PIN 6
#define SPEAKER_PIN 7
#define SWITCH_PIN 9

// Set up the LED number along the strip
#define NUM_LEDS 16

#define COLOR_ORDER GRB
#define VOLTS 5
#define MAX_AMPS 500
#define CHIPSET WS2812B

CRGB leds[NUM_LEDS];
CRGB state[2];

// for buttons
int redValue = 0;
int greenValue = 0;
int blueValue = 0;
int redButtonState = 0;
int blueButtonState = 0;
int greenButtonState = 0;
int MixingState = 0;

// for audio: base RGB - 125
int rgbValues[] = {1, 2, 5};
int valuesToNote[] = {
  0, NOTE_C4, NOTE_E4, NOTE_D4, 0, NOTE_G4, NOTE_A4, NOTE_F4, 0
};
int melody[NUM_LEDS];
const float noteDuration = 250;
const float notePausePar = 1.3;

// for audio controller sensor
CAP1203 sensor;

// state
int currentLed = 0;
int STATE = 0;
bool SPEAKER_TUNRON = true;
bool isTouchingPad = false;
float timer = 0;



void setup() {
 
  Serial.begin(9600);

  Wire.begin();
  
  FastLED.addLeds<CHIPSET, LED_PIN, COLOR_ORDER>(leds, NUM_LEDS);
  FastLED.setMaxPowerInVoltsAndMilliamps(VOLTS, MAX_AMPS);
  FastLED.clear();
  FastLED.show();

  pinMode(BUTTON_RED_PIN,INPUT);
  pinMode(BUTTON_BLUE_PIN,INPUT);
  pinMode(BUTTON_GREEN_PIN,INPUT);
  pinMode(SWITCH_PIN, INPUT);

  
  // Setup sensor
  while (sensor.begin() == false) {
    sensor.begin();
  }
  
  sensor.setPowerButtonPad(PAD_MIDDLE);
  sensor.setSensitivity(SENSITIVITY_2X);
  sensor.setPowerButtonTime(PWR_TIME_280_MS);
  sensor.setPowerButtonEnabled();

  Serial.println("Good");

}


void loop() {
 
   detectingRGB();

//  if (redButtonState == HIGH) Serial.println("red");
//  if (blueButtonState == HIGH) Serial.println("blue");
//  if (greenButtonState == HIGH) Serial.println("green");

  if (STATE == 0 && isPressing()) {
     CRGB newColor = CRGB(redValue, greenValue, blueValue);
     leds[currentLed] = newColor;
     melody[currentLed] = rgbToNote(newColor);
     STATE = 1;
     FastLED.show();
  } 
  
  if (STATE == 1 && !isPressing()) {
     STATE = 0;
     currentLed++;
  }

  if (currentLed == NUM_LEDS) {
    STATE = 2;
    shine();
    reset();
  }
}


void detectingPad() {
  if (!isTouchingPad && sensor.isPowerButtonTouched()) {
    isTouchingPad = true;
    SPEAKER_TUNRON = !SPEAKER_TUNRON;
  } else if (isTouchingPad && !sensor.isPowerButtonTouched()) {
    isTouchingPad = false;
  }
}


void detectingRGB() {
  //reading the vlaue of each light sensor
  redButtonState = digitalRead(BUTTON_RED_PIN);
  blueButtonState = digitalRead(BUTTON_BLUE_PIN);
  greenButtonState = digitalRead(BUTTON_GREEN_PIN);
  

  if (STATE == 0 && isPressing()) {
    delay(50);
    redButtonState = digitalRead(BUTTON_RED_PIN);
    blueButtonState = digitalRead(BUTTON_BLUE_PIN);
    greenButtonState = digitalRead(BUTTON_GREEN_PIN);
  }
  
  
  redValue = redButtonState == HIGH ? 255 : 0;
  blueValue = blueButtonState == HIGH ? 255 : 0;
  greenValue = greenButtonState == HIGH ? 255 : 0;
  
  Serial.println(redValue);
  Serial.println("Blue");
  Serial.println(blueValue);
  Serial.println("Green");
  Serial.println(greenValue);
  

}


bool isPressing() {
  return redButtonState == HIGH || 
        blueButtonState == HIGH || 
        greenButtonState == HIGH;
}


void reset() {
  for (int i = 0; i < NUM_LEDS; i++) {
    playTone(melody[i], noteDuration);
    
    leds[i] = CRGB(0, 0, 0);
    FastLED.show();
    
    melody[i] = 0; 
    delay(noteDuration * notePausePar);
    noTone(SPEAKER_PIN);
  }
  FastLED.clear();
  FastLED.show();
  currentLed = 0;
  STATE = 0;
}


void shine() {
  for (int i = 0; i < NUM_LEDS; i++) {
      int index = NUM_LEDS - i - 1;
      CRGB currColor = leds[index];
      leds[index] *= 0.5;
      FastLED.show();
      delay(50);
      leds[index] = currColor;
  }
}



void playTone(int pitch, float duration) {
  if (sensor.begin() == true) detectingPad();
  if (!SPEAKER_TUNRON) return;
  tone(SPEAKER_PIN, pitch, duration);
}



int rgbToNote (CRGB color) {
  int value = 0;
  value = (color.red / 255) * rgbValues[0] + 
         (color.green / 255) * rgbValues[1] + 
         (color.blue / 255) * rgbValues[2];
  return valuesToNote[value];
}
